// Pretty Printing Visitor over the Abstract Syntax Tree (SLAbsSyn.java)

class PPVisProg implements ProgramVisitor<String> {
   public String visit(Prog p) { return p.body.accept(new PPVisStmList()); }
}

class PPVisStm implements StatementVisitor<String> {
   public String visit(CompoundStatement s) {
     return s.stm1.accept(this)+"\n"+s.stm2.accept(this);
   }
   public String visit(AssignStatement s) {
     return s.id + " = " + s.exp.accept(new PPVisExpr());
   }
   public String visit(PrintStatement s) {
     return "print " + "(" + s.exps.accept(new PPVisExpList()) + ")";
   }
}

class PPVisExpr implements ExpressionVisitor<String> {
   public String visit(IdExpression e) {
     return e.id.toString();
   }
   public String visit(NumExpression e) {
     return e.num.toString();
   }
   public String visit(OpExpression e) {
     return e.left.accept(this) + Utils.opToString(e.op) + e.right.accept(this);
   }
   public String visit(EsepExpression e) {
     return "(" + e.stm.accept(new PPVisStmList()) + "," + e.exp.accept(this) + ")";
   }
}

class PPVisExpList implements ListVisitor<String,Expression> {
   public String visit(NIL<Expression> xs) {
     return "";
   }
   public String visit(CONS<Expression> xs) {
     String sep;
     if (xs.tl instanceof NIL) {
        sep = "";
     } else {
        sep = ", ";
     }
     return xs.hd.accept(new PPVisExpr())+sep+xs.tl.accept(this);
   }
}

class PPVisStmList implements ListVisitor<String,Statement> {
  public String visit(NIL<Statement> xs) {
     return "";
   }
   public String visit(CONS<Statement> xs) {
     return xs.hd.accept(new PPVisStm())+((xs.tl instanceof NIL)?"":";\n")+xs.tl.accept(this);
   }
}
