/* Copyright (c) 1997 Andrew W. Appel.  Licensed software: see LICENSE file */
//package straightparse;

import java_cup.runtime.Symbol;

%% 

%public
%class SLLex
%implements java_cup.runtime.Scanner
%function next_token
%type java_cup.runtime.Symbol
%char

%{
private int stringstart;

private StringBuffer charbuf = null;

private void newline() {
  errorMsg.newline(yychar);
}

private void err(int pos, String s) {
  errorMsg.error(pos,s);
}

private void err(String s) {
  err(yychar,s);
}

private Symbol tok(int kind) {
    return new Symbol(kind, yychar, yychar+yylength());
}

private int comLevel = 0;
private errorMsg.ErrorMsg errorMsg;

SLLex(java.io.InputStream s, errorMsg.ErrorMsg e) {
  this(s);
  errorMsg=e;
}

public static String toBString(java_cup.runtime.Symbol s) {
    switch (s.sym) {
	case SLParse_sym.COMMA: return "," ;
	case SLParse_sym.SEMICOLON: return ";" ;
	case SLParse_sym.LPAREN: return "(" ;
	case SLParse_sym.RPAREN: return ")" ;
	case SLParse_sym.PLUS: return "+" ;
	case SLParse_sym.MINUS: return "-" ;
	case SLParse_sym.TIMES: return "*" ;
	case SLParse_sym.DIVIDE: return "/" ;
	case SLParse_sym.EQ: return "=" ;
	case SLParse_sym.PRINT: return "print" ;	
	case SLParse_sym.EOF: return "EOF" ;
	case SLParse_sym.ID: return "ID "+s.value.toString();
	case SLParse_sym.INT: return "INT "+s.value.toString();
	default: return "???";
    }
}

%}

%eofval{
	{if (comLevel>0) err (stringstart, "unclosed comment");
	 return tok(SLParse_sym.EOF);
        }
%eofval}       


%state A B
idchars=[A-Za-z_0-9]
id=[A-Za-z]{idchars}*
ws=("\012"|[\t\ ])*
%%
<YYINITIAL>{ws}	{}
<YYINITIAL>\n	{newline();}
<YYINITIAL>","	{return tok(SLParse_sym.COMMA);}
<YYINITIAL>";"	{return tok(SLParse_sym.SEMICOLON);}
<YYINITIAL>"("	{return tok(SLParse_sym.LPAREN);}
<YYINITIAL>")"	{return tok(SLParse_sym.RPAREN);}
<YYINITIAL>"+"	{return tok(SLParse_sym.PLUS);}
<YYINITIAL>"-"	{return tok(SLParse_sym.MINUS);}
<YYINITIAL>"*"	{return tok(SLParse_sym.TIMES);}
<YYINITIAL>"/"	{return tok(SLParse_sym.DIVIDE);}
<YYINITIAL>"="	{return tok(SLParse_sym.EQ);}
<YYINITIAL>"print"	{return tok(SLParse_sym.PRINT);}
<YYINITIAL>{id}	{return new Symbol(SLParse_sym.ID,
				yychar,yychar+yylength(),
				     yytext());}
<YYINITIAL>[0-9]+ {return new Symbol(SLParse_sym.INT,
			yychar,yychar+yylength(),
			  new Integer(Integer.parseInt(yytext())));}
<YYINITIAL>"/*"	{yybegin(A); stringstart = yychar; comLevel = 1;}
<YYINITIAL>"*/"	{err("unmatched close comment");}
<YYINITIAL>"//" {yybegin(B);}
<B>[^\n]+       {/* ignore */}
<B>\n		{yybegin(YYINITIAL); }
<YYINITIAL>.	{err("illegal token");}
<A>"/*"		{comLevel++;}
<A>\n		{newline();}
<A>"*/" 	{comLevel--; if (comLevel==0) yybegin(YYINITIAL);}
<A>.		{}




