/*
-- $Id: SLParse.y,v 1.2 2005/11/09 19:06:37 hwloidl Exp $
--
-- StraightLine Interpreter
--  Parser
-- Practical on compiler design, LMU 2005/6
-- Based on Appels book on "Modern Compiler Implementation"
--
-- For maps could consider:
--   http://java.sun.com/j2se/1.5.0/docs/api/java/util/Map.html
---------------------------------------------------------------------------
*/

//package straightparse;

import java_cup.runtime.*;

/* Preliminaries to set up and use the scanner.  */
/*
init with {: java_cup.runtime.Scanner.scanner.init();              :};
scan with {: return java_cup.runtime.Scanner.scanner.next_token(); :};
*/

parser code  {: 

  public void syntax_error(java_cup.runtime.Symbol token) {
   System.out.println("Syntax error at token " + SLLex.toBString(token)); 
  } 
 
    /*
  public void syntax_error(java_cup.runtime.Symbol current) {
   report_error("Syntax error (" + current.sym + ")", current);
  }

  errorMsg.ErrorMsg errorMsg;

  public void report_error(String message, java_cup.runtime.Symbol info) {
      errorMsg.error(info.left, message);
  }

  public void report_error(String message, Object info) {
    if (info instanceof java_cup.runtime.Symbol)
	report_error(message,(java_cup.runtime.Symbol)info);
    else errorMsg.error(0, message);
  }
    */
:};


/* Terminals (tokens returned by the scanner). */
terminal String ID;
terminal Integer INT;
terminal PRINT; 
terminal EQ;
terminal PLUS;
terminal MINUS;
terminal TIMES;
terminal DIVIDE;
terminal COMMA;
terminal LPAREN;
terminal RPAREN;
terminal SEMICOLON;
// terminal EOF; // EOF is created automatically

/* Non terminals */
non terminal Program Prg;
non terminal List<Statement> StmList;
non terminal Statement Stm;
non terminal Expression Expr;
non terminal List<Expression> ExprList;
non terminal Binop BinOp;

/* Precedences */
precedence left SEMICOLON;
precedence left EQ;
precedence left PRINT;
precedence left PLUS, MINUS;
precedence left TIMES, DIVIDE;

/* Top level symbol of the grammar */
start with Prg;

/* The grammar */

Prg ::= StmList:ss            {: RESULT = new Prog(Utils.reverse(ss)); :}
    ;

StmList ::= Stm:s                {: RESULT = new CONS<Statement>(s,new NIL<Statement>()); :}
          | StmList:ss SEMICOLON Stm:s {: RESULT = new CONS<Statement>(s,ss); :}
          ; 

Stm ::= ID:id EQ Expr:e                   {: RESULT = new AssignStatement(id,e); :}
      | PRINT LPAREN ExprList:es RPAREN   {: RESULT = new PrintStatement(Utils.reverse(es)); :}
      ;

BinOp ::=
      PLUS    			       {: RESULT = new PLUS() ; :}
    | MINUS   			       {: RESULT = new MINUS() ; :}
    | TIMES   			       {: RESULT = new TIMES() ; :}
    | DIVIDE  			       {: RESULT = new DIVIDE() ; :}
    ;

Expr ::= Expr:e1 BinOp:op Expr:e2  {: RESULT = new OpExpression(e1,op,e2); :}
    | ID:id                  {: RESULT = new IdExpression(id); :}
    | INT:num                 {: RESULT = new NumExpression(num); :}
    | LPAREN Expr:e RPAREN    {: RESULT = e; :}
    | LPAREN StmList:ss COMMA Expr:e RPAREN {: RESULT = new EsepExpression(Utils.reverse(ss),e); :}
    ;
   
ExprList ::= Expr:e                    {: RESULT = new CONS<Expression>(e,new NIL<Expression>());:}
           | ExprList:es COMMA Expr:e  {: RESULT = new CONS<Expression>(e,es); :}
           ;
/* EOF */
